#!-------------------------------------------------------------------------------#!
#!
#!    MemForm              : Data entry only, no file update
#!
#!------------------------------------------------------------------------------
#!
#PROCEDURE(MemForm,'Data entry only, no file update'),SCREEN,PULLDOWN
#!------------------------------------------------------------------------------
#!
#!                           The MemForm Template
#!
#! Created  by Mander Li   Sept 93   for release 3.004
#! Modified by Mander Li   Nov  93   for release 3.006
#!
#! This is a 1. Memory Form in Clarion 2.1 terminology
#!           2. Screen template with Ok and Cancel buttons
#!           3. Form template without file updates
#!
#! The MemForm can be used for:
#! 1. prompting for parameters before an operation (eg printing a report)
#! 2. getting user's confirmation before an opeartion
#! 3. displaying a message (in this case remove the Cancel button)
#!
#! Note that in this template,
#! 1. The Cancel button can be removed.
#! 2. The OK button cannot be removed.
#! 3. The primary file is optional.  Even if the primary file is specified,
#!    the procedure will not open, read, write, or close the file.
#! 4. The secondary files, if specified, will be read for field lookups
#!    and validations.
#!
#! This template was created in the following steps:
#! 1. copy the Form template
#! 2. remove those line related to file updates and the Insert, Change, Delete
#!    buttons
#! 3. fix the problem that the generated procedure has compilation error
#!    (Unknown identifier: ?Cancel) if the Cancel button is removed.
#!    Those lines changed are marked with #!<<<
#!
#!------------------------------------------------------------------------------
#PROTOTYPE('')
#PROMPT('Next Procedure ',PROCEDURE),%NextProcedure
#INSERT(%StandardHeader)
#INSERT(%InitFormSymbols)
%Procedure      PROCEDURE                        #<! Template = MemForm

%LocalData

#IF(%CloseFiles)
#INSERT(%CloseFilesFlags)                        #!Primary or Secondary Opened
#SET(%SetFile,%Primary)
#INSERT(%CloseRelatedFlags)                      #!Related file Opened
#ENDIF
NoMoreFields     BYTE(0)                         !No more fields flag
%ScreenStructure
%PullDownStructure
#EMBED('Data Section')

  CODE

  #EMBED('Setup Procedure')
  #INSERT(%OpenSecondaryFiles)                   #!Group from Clarion9.tpx
  #INSERT(%FieldLookupOpen)                      #!Check for Must be in File

  #FOR(%Formula)
    #IF(UPPER(%FormulaClass) = 'SETUP')
  #INSERT(%GenerateFormula)
    #ENDIF
  #ENDFOR
  #IF(%SecondaryExist)                           #<!IF schema has a Secondary
  DO SecondaryLookups                            !Read any lookup fields
  #ENDIF
  #IF(%PullDownStructure)
  OPEN(%PullDown)
  #ENDIF
  OPEN(Screen)                                   !Open the FORM screen
  #EMBED('Setup Screen')
  DISPLAY                                        !Display screen fields

  LOOP                                           !Begin Main process loop
    #IF(%SecondaryExist)                         #<!IF File schema has Secondary
    #INSERT(%SecondaryChanged)
    #ENDIF
    #IF(%LoopFormulasExist = 'TRUE')             #<!Are there Formula fields?
     #SET(%GenerateFormulasOn,'TRUE')
    DO FormulaFields                             !Calculate Formula fields
    #ENDIF

    CASE SELECTED()                              !Process selected Field
    #INSERT(%ScreenSetupRoutines)
      OF NoMoreFields                            !User pressed Enter or OK
        #IF(%NextProcedure)
        #EMBED('Setup Next Procedure')
        %NextProcedure                           #<!Call the Next Procedure
        #EMBED('Return from Next Procedure')
        #ENDIF
        BREAK                                    !Break from main Loop
    END                                          !End CASE Selected()

    ACCEPT                                       !Enable screen entry
    CASE KEYCODE()
      #FOR(%ScreenField)                         #!<<<
        #IF(%ScreenFieldUse = '?Cancel')         #!<<<
      OF EscKey                                  !User pressed Escape key
        IF FIELD() <> ?Cancel                    !If user pressed Escape
          SELECT(?Cancel)                        !Select Cancel button
          PRESS(EnterKey)                        !Process Cancel button code
          CYCLE                                  !Cycle to Accept
        END                                      !Field was not Cancel button
        #ENDIF                                   #!<<<
      #ENDFOR                                    #!<<<
    #IF(%HotKeysExist)
    #FOR(%HotKey)
      OF %HotKey                                 !User defined HotKey
        %HotKeyProc                              !HotKey Procedure
    #ENDFOR
    #ENDIF
    END                                          !End CASE Keycode

    CASE FIELD()                                 !Process fields
    #FOR(%ScreenField)
      #IF(%ScreenFieldUse = '?Ok')
      OF ?Ok                                     !On the OK button
        #IF(%ScreenFieldEdit <> %NULL)
        %ScreenFieldEdit                         #<!Field Edit procedure
        #ENDIF
        SELECT(1)                                !Start with the first field
        SELECT                                   !and cycle non-stop
        CYCLE                                    !restart main process loop

      #ELSIF(%ScreenFieldUse = '?Cancel')
      OF ?Cancel                                 !On Cancel button
        #IF(%ScreenFieldEdit <> %NULL)
        %ScreenFieldEdit                         #<!Field edit procedure
        #ENDIF
        BREAK                                    !Break from main LOOP

      #ELSE
      #INSERT(%FormEditRoutines)
      #ENDIF
    #ENDFOR
    #FOR(%PulldownField)                         #! add all procedure or
     #IF(%PulldownFieldType = 'PROCEDURE')       #! source code calls
      OF %PulldownField                         #<!For a Pulldown field
        %PulldownFieldProc                      #<!  execute its procedure
     #ENDIF
    #ENDFOR
    END                                          !End CASE FIELD
  END                                            !END MAIN PROCESS LOOP

  #FOR(%Formula)
    #IF(UPPER(%FormulaClass) = 'RETURN')
  #INSERT(%GenerateFormula)                      #<!Return Class formula
    #ENDIF
  #ENDFOR

  #IF(%PullDownStructure)
  CLOSE(%PullDown)
  #ENDIF
  #IF(%CloseFiles)
  #INSERT(%CloseOpenedFiles)                     #<!Close files Opened here
  #SET(%SetFile,%Primary)
  #INSERT(%CloseRelatedFiles)                    #<!Close related Opened here
  #ENDIF
#EMBED('End of Procedure ')

#INSERT(%InitFields)
#INSERT(%GenFormulas)
#IF(%SecondaryExist)
#INSERT(%SecondaryLookups)
#ENDIF
